@@ -244,6 +244,11 @@ enum Opt {
         /// Use as `--rustc-extra-args="--my-arg"`
         #[structopt(long = "rustc-extra-args")]
         rustc_extra_args: Vec<String>,
+        /// Install extra requires aka. optional dependencies
+        ///
+        /// Use as `--extras=extra1,extra2`
+        #[structopt(short = "E", long, use_delimiter = true, multiple = false)]
+        extras: Vec<String>,
     },
     /// Build only a source distribution (sdist) without compiling.
     ///
@@ -318,6 +323,9 @@ enum Pep517Command {
         /// Strip the library for minimum file size
         #[structopt(long)]
         strip: bool,
+        /// Build editable wheels
+        #[structopt(long)]
+        editable: bool,
     },
     /// The implementation of build_sdist
     #[structopt(name = "write-sdist")]
@@ -351,7 +359,7 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
                 build_options.interpreter.as_ref(),
                 Some(version) if version.len() == 1
             ));
-            let context = build_options.into_build_context(true, strip)?;
+            let context = build_options.into_build_context(true, strip, false)?;
 
             // Since afaik all other PEP 517 backends also return linux tagged wheels, we do so too
             let tags = match context.bridge {
@@ -379,8 +387,9 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
         Pep517Command::BuildWheel {
             build_options,
             strip,
+            editable,
         } => {
-            let build_context = build_options.into_build_context(true, strip)?;
+            let build_context = build_options.into_build_context(true, strip, editable)?;
             let wheels = build_context.build_wheels()?;
             assert_eq!(wheels.len(), 1);
             println!("{}", wheels[0].0.to_str().unwrap());
@@ -438,7 +447,9 @@ fn upload_ui(items: &[PathBuf], publish: &PublishOpt) -> Result<()> {
                             println!("🔑 Removed wrong password from keyring")
                         }
                         Err(KeyringError::NoPasswordFound) | Err(KeyringError::NoBackendFound) => {}
-                        Err(err) => eprintln!("⚠ Failed to remove password from keyring: {}", err),
+                        Err(err) => {
+                            eprintln!("⚠️ Warning: Failed to remove password from keyring: {}", err)
+                        }
                     }
                 }
 
@@ -448,8 +459,8 @@ fn upload_ui(items: &[PathBuf], publish: &PublishOpt) -> Result<()> {
                 let filename = i.file_name().unwrap_or_else(|| i.as_os_str());
                 if let UploadError::FileExistsError(_) = err {
                     if publish.skip_existing {
-                        eprintln!(
-                            "⚠  Skipping {:?} because it appears to already exist",
+                        println!(
+                            "⚠️ Note: Skipping {:?} because it appears to already exist",
                             filename
                         );
                         continue;
@@ -476,7 +487,10 @@ fn upload_ui(items: &[PathBuf], publish: &PublishOpt) -> Result<()> {
             Ok(()) => {}
             Err(KeyringError::NoBackendFound) => {}
             Err(err) => {
-                eprintln!("⚠ Failed to store the password in the keyring: {:?}", err);
+                eprintln!(
+                    "⚠️ Warning: Failed to store the password in the keyring: {:?}",
+                    err
+                );
             }
         }
     }
@@ -497,7 +511,7 @@ fn run() -> Result<()> {
             strip,
             no_sdist,
         } => {
-            let build_context = build.into_build_context(release, strip)?;
+            let build_context = build.into_build_context(release, strip, false)?;
             if !no_sdist {
                 build_context.build_source_distribution()?;
             }
@@ -511,7 +525,7 @@ fn run() -> Result<()> {
             no_strip,
             no_sdist,
         } => {
-            let build_context = build.into_build_context(!debug, !no_strip)?;
+            let build_context = build.into_build_context(!debug, !no_strip, false)?;
 
             if !build_context.release {
                 eprintln!("⚠️  Warning: You're publishing debug wheels");
@@ -544,6 +558,7 @@ fn run() -> Result<()> {
             rustc_extra_args,
             release,
             strip,
+            extras,
         } => {
             let venv_dir = match (env::var_os("VIRTUAL_ENV"), env::var_os("CONDA_PREFIX")) {
                 (Some(dir), None) => PathBuf::from(dir),
@@ -569,6 +584,7 @@ fn run() -> Result<()> {
                 &venv_dir,
                 release,
                 strip,
+                extras,
             )?;
         }
         Opt::SDist { manifest_path, out } => {
