@@ -1,4 +1,5 @@
 use crate::auditwheel::PlatformTag;
+use crate::cross_compile::is_cross_compiling;
 use crate::{BridgeModel, Target};
 use anyhow::{bail, format_err, Context, Result};
 use regex::Regex;
@@ -7,7 +8,7 @@ use std::collections::HashSet;
 use std::fmt;
 use std::io;
 use std::path::{Path, PathBuf};
-use std::process::Command;
+use std::process::{Command, Stdio};
 use std::str;
 
 /// This snippets will give us information about the python interpreter's
@@ -16,6 +17,7 @@ const GET_INTERPRETER_METADATA: &str = include_str!("get_interpreter_metadata.py
 const MINIMUM_PYTHON_MINOR: usize = 6;
 /// Be liberal here to include preview versions
 const MAXIMUM_PYTHON_MINOR: usize = 12;
+const MAXIMUM_PYPY_MINOR: usize = 8;
 
 /// Identifies conditions where we do not want to build wheels
 fn windows_interpreter_no_build(
@@ -93,7 +95,7 @@ fn find_all_windows(target: &Target, min_python_minor: usize) -> Result<Vec<Stri
     // which is used to find the installed interpreters
     let execution = Command::new("py").arg("-0").output();
     if let Ok(output) = execution {
-        let expr = Regex::new(r" -(\d).(\d)-(\d+)(?: .*)?").unwrap();
+        let expr = Regex::new(r" -(\d).(\d+)-(\d+)(?: .*)?").unwrap();
         let lines = str::from_utf8(&output.stdout).unwrap().lines();
         for line in lines {
             if let Some(capture) = expr.captures(line) {
@@ -165,7 +167,12 @@ fn find_all_windows(target: &Target, min_python_minor: usize) -> Result<Vec<Stri
         }
 
         for path in paths {
-            let executable = Path::new(&path).join("python");
+            let executable_win = Path::new(&path).join("python.exe");
+            let executable = if executable_win.exists() {
+                executable_win
+            } else {
+                Path::new(&path).join("python")
+            };
             let python_info = Command::new(&executable)
                 .arg("-c")
                 .arg("import sys; print(sys.version)")
@@ -245,7 +252,10 @@ struct IntepreterMetadataMessage {
     abiflags: Option<String>,
     interpreter: String,
     ext_suffix: Option<String>,
+    // comes from `sysconfig.get_platform()`
     platform: String,
+    // comes from `platform.system()`
+    system: String,
     abi_tag: Option<String>,
     base_prefix: String,
 }
@@ -280,6 +290,10 @@ pub struct PythonInterpreter {
     pub abi_tag: Option<String>,
     /// We need this value for windows abi3 linking
     pub libs_dir: PathBuf,
+    /// Comes from `sysconfig.get_platform()`
+    ///
+    /// Note that this can be `None` when cross compiling
+    pub platform: Option<String>,
 }
 
 /// Returns the abiflags that are assembled through the message, with some
@@ -293,10 +307,13 @@ fn fun_with_abiflags(
     target: &Target,
     bridge: &BridgeModel,
 ) -> Result<String> {
-    if bridge != &BridgeModel::Cffi && target.get_python_os() != message.platform {
+    if bridge != &BridgeModel::Cffi
+        && target.get_python_os() != message.system
+        && !is_cross_compiling(target)?
+    {
         bail!(
-            "sys.platform in python, {}, and the rust target, {:?}, don't match ಠ_ಠ",
-            message.platform,
+            "platform.system() in python, {}, and the rust target, {:?}, don't match ಠ_ಠ",
+            message.system,
             target,
         )
     }
@@ -312,11 +329,11 @@ fn fun_with_abiflags(
     if message.interpreter == "pypy" {
         // pypy does not specify abi flags
         Ok("".to_string())
-    } else if message.platform == "windows" {
-        if message.abiflags.is_some() {
-            bail!("A python 3 interpreter on windows does not define abiflags in its sysconfig ಠ_ಠ")
-        } else {
+    } else if message.system == "windows" {
+        if matches!(message.abiflags.as_deref(), Some("") | None) {
             Ok("".to_string())
+        } else {
+            bail!("A python 3 interpreter on windows does not define abiflags in its sysconfig ಠ_ಠ")
         }
     } else if let Some(ref abiflags) = message.abiflags {
         if message.minor >= 8 {
@@ -340,9 +357,17 @@ impl PythonInterpreter {
     ///
     /// If abi3 is true, cpython wheels use the generic abi3 with the given version as minimum
     pub fn get_tag(&self, platform_tag: PlatformTag, universal2: bool) -> String {
+        let platform = if self.target.is_windows() {
+            // Restrict `sysconfig.get_platform()` usage to Windows only for now
+            // so we don't need to deal with macOS deployment target
+            self.platform
+                .clone()
+                .unwrap_or_else(|| self.target.get_platform_tag(platform_tag, universal2))
+        } else {
+            self.target.get_platform_tag(platform_tag, universal2)
+        };
         match self.interpreter_kind {
             InterpreterKind::CPython => {
-                let platform = self.target.get_platform_tag(platform_tag, universal2);
                 if self.target.is_unix() {
                     format!(
                         "cp{major}{minor}-cp{major}{minor}{abiflags}-{platform}",
@@ -362,7 +387,6 @@ impl PythonInterpreter {
                 }
             }
             InterpreterKind::PyPy => {
-                let platform = self.target.get_platform_tag(platform_tag, universal2);
                 // pypy uses its version as part of the ABI, e.g.
                 // pypy 3.7 7.3 => numpy-1.20.1-pp37-pypy37_pp73-manylinux2010_x86_64.whl
                 format!(
@@ -461,8 +485,8 @@ impl PythonInterpreter {
         match message.interpreter.as_str() {
             "cpython" => interpreter = InterpreterKind::CPython,
             "pypy" => interpreter = InterpreterKind::PyPy,
-            _ => {
-                bail!("Invalid interpreter");
+            other => {
+                bail!("Unsupported interpreter {}", other);
             }
         };
 
@@ -471,6 +495,23 @@ impl PythonInterpreter {
             executable.as_ref().display()
         ))?;
 
+        let platform = if message.platform.starts_with("macosx") {
+            // macOS contains system version in platform name so need special handle
+            let mut parts = message.platform.splitn(3, '-');
+            let err_ctx = "Invalid platform string from `sysconfig.get_platform()`";
+            let prefix = parts.next().context(err_ctx)?;
+            let base_version = parts.next().context(err_ctx)?;
+            let suffix = parts.next().context(err_ctx)?;
+            let (major_ver, minor_ver) = base_version.split_once('.').context(err_ctx)?;
+            let major_ver: u64 = major_ver.parse()?;
+            let minor_ver = if major_ver > 10 { "0" } else { minor_ver };
+            // FIXME: We should also take MACOSX_DEPLOYMENT_TARGET env var into account
+            format!("{}_{}_{}_{}", prefix, major_ver, minor_ver, suffix)
+        } else {
+            message.platform
+        };
+        let platform = platform.to_lowercase().replace('-', "_").replace('.', "_");
+
         Ok(Some(PythonInterpreter {
             major: message.major,
             minor: message.minor,
@@ -483,6 +524,7 @@ impl PythonInterpreter {
             interpreter_kind: interpreter,
             abi_tag: message.abi_tag,
             libs_dir: PathBuf::from(message.base_prefix).join("libs"),
+            platform: Some(platform),
         }))
     }
 
@@ -497,9 +539,16 @@ impl PythonInterpreter {
         let executables = if target.is_windows() {
             find_all_windows(target, min_python_minor)?
         } else {
-            (min_python_minor..MAXIMUM_PYTHON_MINOR)
+            let mut executables: Vec<String> = (min_python_minor..MAXIMUM_PYTHON_MINOR)
                 .map(|minor| format!("python3.{}", minor))
-                .collect()
+                .collect();
+            // Also try to find PyPy for cffi and pyo3 bindings
+            if matches!(bridge, BridgeModel::Cffi) || bridge.is_bindings("pyo3") {
+                executables.extend(
+                    (min_python_minor..MAXIMUM_PYPY_MINOR).map(|minor| format!("pypy3.{}", minor)),
+                );
+            }
+            executables
         };
         let mut available_versions = Vec::new();
         for executable in executables {
@@ -512,7 +561,7 @@ impl PythonInterpreter {
         Ok(available_versions)
     }
 
-    /// Checks that given list of executables are all valid python intepreters,
+    /// Checks that given list of executables are all valid python interpreters,
     /// determines the abiflags and versions of those interpreters and
     /// returns them as [PythonInterpreter]
     pub fn check_executables(
@@ -538,6 +587,45 @@ impl PythonInterpreter {
 
         Ok(available_versions)
     }
+
+    /// Run a python script using this Python interpreter.
+    pub fn run_script(&self, script: &str) -> Result<String> {
+        let out = Command::new(&self.executable)
+            .env("PYTHONIOENCODING", "utf-8")
+            .stdin(Stdio::piped())
+            .stdout(Stdio::piped())
+            .stderr(Stdio::inherit())
+            .spawn()
+            .and_then(|mut child| {
+                use std::io::Write;
+                child
+                    .stdin
+                    .as_mut()
+                    .expect("piped stdin")
+                    .write_all(script.as_bytes())?;
+                child.wait_with_output()
+            });
+
+        match out {
+            Err(err) => {
+                if err.kind() == io::ErrorKind::NotFound {
+                    bail!(
+                        "Could not find any interpreter at {}, \
+                     are you sure you have Python installed on your PATH?",
+                        self.executable.display()
+                    );
+                } else {
+                    bail!(
+                        "Failed to run the Python interpreter at {}: {}",
+                        self.executable.display(),
+                        err
+                    );
+                }
+            }
+            Ok(ok) if !ok.status.success() => bail!("Python script failed"),
+            Ok(ok) => Ok(String::from_utf8(ok.stdout)?),
+        }
+    }
 }
 
 impl fmt::Display for PythonInterpreter {
